/*=====================================================================
profiler.cpp
------------
File created by ClassTemplate on Thu Feb 24 19:00:30 2005

Copyright (C) Nicholas Chapman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html..
=====================================================================*/
#include "profiler.h"


#include "../utils/stringutils.h"
#include <process.h>
#include <iostream>
#include <assert.h>
#include <winnt.h>
#include <dbghelp.h>


Profiler::Profiler(HANDLE target_process_, HANDLE target_thread_)
:	target_thread(target_thread_),
	target_process(target_process_)
{
	
}


Profiler::~Profiler()
{
	
}


void Profiler::sampleTarget()
{
	// An open question is whther or not this routine can be called
	// reentrantly by the multi-media timer support.

	HRESULT result = SuspendThread(target_thread);

	if(result == 0xffffffff)
		throw ProfilerExcep("SuspendThread failed.");

	CONTEXT threadcontext;
	threadcontext.ContextFlags = CONTEXT_i386 | CONTEXT_CONTROL;
	result = GetThreadContext(target_thread, &threadcontext);
	if(!result)
		throw ProfilerExcep("GetThreadContext failed.");

  int stacklevels = 0;
	PROFILER_ADDR walk[256];
  walk[stacklevels++] = threadcontext.Eip;

	STACKFRAME64 frame;
	memset(&frame, 0, sizeof(frame));

	frame.AddrStack.Offset = threadcontext.Esp;
	frame.AddrPC.Offset = threadcontext.Eip;
	frame.AddrFrame.Offset = threadcontext.Ebp;
	frame.AddrStack.Mode = frame.AddrPC.Mode = frame.AddrFrame.Mode = AddrModeFlat;

  while(true)
	{
		BOOL result = StackWalk64(
			IMAGE_FILE_MACHINE_I386,
			target_process,
			target_thread,
			&frame,
			&threadcontext,
			NULL,
			&SymFunctionTableAccess64,
			&SymGetModuleBase64,
			NULL
			);

		if (!result || stacklevels >= 256)
      break;

    walk[stacklevels++] = (PROFILER_ADDR)frame.AddrReturn.Offset;
	}

	//std::cout << "addr: " << addr << std::endl;

	result = ResumeThread(target_thread);
	if(!result)
		throw ProfilerExcep("ResumeThread failed.");

	//NOTE: this has to go after ResumeThread.  Otherwise mem allocation needed by std::map
	//may hit a lock held by the suspended thread.

	flatcounts[walk[0]]++;

	CallStack stack;
  for (int n=0;n<stacklevels;n++)
    stack.addr.push_back(walk[n]);
  callstacks[stack]++;
}

// returns true if the target thread has finished
bool Profiler::targetExited() const
{
	DWORD code = WaitForSingleObject(target_thread, 100);
	return (code == WAIT_OBJECT_0);
}


//void Profiler::saveIPs(std::ostream& stream)
//{
//	for(std::map<Sample, int>::const_iterator i = counts.begin(); 
//		i != counts.end(); ++i)
//	{
//		const Sample &sample = i->first;
//		int count = i->second;
//		stream << ::toHexString(sample.addr) << " " << count << "\n";
//	}
//
//	stream.flush();
//}
//
